/**
 * @copyright 2017-present, Charlike Mike Reagent <olsten.larck@gmail.com>
 * @license Apache-2.0
 */

module.exports = function parseGitCommit (commitMessage, mapFn) {
  const mapper = typeof mapFn === 'function' ? mapFn : (c) => c;
  const lines = commitMessage.split('\n');
  const header = lines.shift();
  const body = lines.shift() || null;
  const footer = lines.join('\n').trim() || null;
  const parts = /^(\w+)(?:\((.+)\))?: (.+)$/.exec(header);

  if (!parts) {
    throw new Error('invalid commit message: <type>[optional scope]: <description>');
  }

  const [type, scope, subject] = parts.slice(1);

  const commit = {
    type,
    scope,
    subject,
    header,
    body,
    footer,
  };

  return mapper(commit);
};
